package com.steerinfo.dil.service.impl;

import com.fasterxml.jackson.databind.node.BigIntegerNode;
import com.steerinfo.dil.mapper.BmstruckCheckMapper;
import com.steerinfo.dil.mapper.BmstruckDetailsOrderMapper;
import com.steerinfo.dil.mapper.BmstruckDetailsStatementMapper;
import com.steerinfo.dil.model.BmstruckCheck;
import com.steerinfo.dil.model.BmstruckDetailsOrder;
import com.steerinfo.dil.model.BmstruckDetailsStatement;
import com.steerinfo.dil.util.NoUtil;
import com.steerinfo.framework.mapper.IBaseMapper;
import com.steerinfo.framework.service.impl.BaseServiceImpl;
import com.steerinfo.dil.model.BmstruckStatement;
import com.steerinfo.dil.mapper.BmstruckStatementMapper;
import com.steerinfo.dil.service.IBmstruckStatementService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.DecimalFormat;
import java.util.Date;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * BmstruckStatement服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-10-15 06:20
 * 类描述
 * 修订历史：
 * 日期：2021-10-15
 * 作者：generator
 * 参考：
 * 描述：BmstruckStatement服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "bmstruckStatementService")
public class BmstruckStatementServiceImpl implements IBmstruckStatementService {

    @Autowired
    BmstruckStatementMapper bmstruckStatementMapper;
    @Autowired
    BmstruckDetailsStatementMapper bmstruckDetailsStatementMapper;
    @Autowired
    BmstruckDetailsOrderMapper bmstruckDetailsOrderMapper;
    @Autowired
    BmstruckCheckMapper bmstruckCheckMapper;
    @Autowired
    NoUtil noUtil;

    /**
     * 展示计费账单
     * @param mapValue
     * @return
     */
    @Override
    public List<Map<String, Object>> getTruckStatementList(Map<String, Object> mapValue) {
        List<Map<String, Object>> mapList = bmstruckStatementMapper.getTruckStatementList(mapValue);
        String checkStatus = "";
        for (Map<String,Object> map: mapList ) {
            BigDecimal checkCarrierStatus = (BigDecimal) map.get("checkCarrierStatus");
            BigDecimal checkReceiveCustomerStatus = (BigDecimal) map.get("checkReceiveCustomerStatus");
            BigDecimal checkShipperStatus = (BigDecimal) map.get("checkShipperStatus");
            // 有人拒绝：未通过审核
            if (checkCarrierStatus.intValue() == 0 || checkReceiveCustomerStatus.intValue() == 0 || checkShipperStatus.intValue() == 0){
                checkStatus = "未通过审核";
            }
            // 全部通过：已通过审核
            if (checkCarrierStatus.intValue() == 1 && checkReceiveCustomerStatus.intValue() == 1 && checkShipperStatus.intValue() == 1) {
                checkStatus = "已通过审核";
            }
            // 有人未审核，且没人拒绝：审核中
            if (checkCarrierStatus == null || checkReceiveCustomerStatus == null || checkShipperStatus == null && checkStatus != "未通过审批"){
                checkStatus = "审核中";
            }
            // 全部都没审核：待审核
            if (checkCarrierStatus == null && checkReceiveCustomerStatus == null && checkShipperStatus == null) {
                checkStatus = "待审核";
            }
            map.put("checkStatus",checkStatus);
        }
        return mapList;
    }

    /**
     * 生成汽运账单
     * @param mapList
     * @return
     */
    @Transactional
    @Override
    public int addTruckStatement(List<Map<String, Object>> mapList) {
        BmstruckStatement bmstruckStatement = new BmstruckStatement();
        BmstruckCheck bmstruckCheck = new BmstruckCheck();
        // 汽运账单主键
        BigDecimal statementId = bmstruckStatementMapper.selectStatementId();
        bmstruckStatement.setStatementId(statementId);
        BigDecimal checkId = bmstruckCheckMapper.selectCheckId();
        bmstruckCheck.setCheckId(checkId);
        bmstruckCheck.setStatementId(statementId);
        bmstruckCheckMapper.insertSelective(bmstruckCheck);
        // 汽运账单编号
        String statementNumber = noUtil.setResultNo("QYZD",statementId);
        bmstruckStatement.setStatementNumber(statementNumber);
        // 支付单位
        bmstruckStatement.setStatementPaymentUnit("四川达州钢铁集团有限责任公司");
        // 备注
        bmstruckStatement.setStatementRemark("汽运费账单");
        // 常规字段
        bmstruckStatement.setInsertTime(new Date());
        bmstruckStatement.setInsertUsername("admin");
        bmstruckStatement.setUpdateTime(new Date());
        bmstruckStatement.setUpdateUsername("admin");
        bmstruckStatement.setInsertUpdateRemark("无");
        Double doubleTotalAmount = new Double(0);
        for (Map<String,Object> map: mapList ) {
            // 中间表主键
            BigDecimal detailsStatementId = bmstruckDetailsStatementMapper.selectDetailsStatementId();
            // 详单id
            BigDecimal detailsId = new BigDecimal((Integer) map.get("detailsId"));
            BmstruckDetailsOrder bmstruckDetailsOrder = bmstruckDetailsOrderMapper.selectByPrimaryKey(detailsId);
            bmstruckDetailsOrder.setWetherToStatement(new BigDecimal(1));
            bmstruckDetailsOrderMapper.updateByPrimaryKeySelective(bmstruckDetailsOrder);
            BmstruckDetailsStatement bmstruckDetailsStatement = new BmstruckDetailsStatement();
            bmstruckDetailsStatement.setDetailsStatementId(detailsStatementId);
            bmstruckDetailsStatement.setStatementId(statementId);
            bmstruckDetailsStatement.setDetailsId(detailsId);
            // 设置常规字段
            bmstruckDetailsStatement.setInsertTime(new Date());
            bmstruckDetailsStatement.setInsertUsername("admin");
            bmstruckDetailsStatement.setUpdateTime(new Date());
            bmstruckDetailsStatement.setUpdateUsername("admin");
            bmstruckDetailsStatement.setInsertUpdateRemark("无");
            BigDecimal detailsAmount = new BigDecimal((Integer)map.get("detailsAmount"));
            Double detailsAmountDouble = Double.valueOf(detailsAmount.toString());
            doubleTotalAmount += detailsAmountDouble;
            bmstruckDetailsStatementMapper.insertSelective(bmstruckDetailsStatement);
        }
        DecimalFormat df = new DecimalFormat("0.00");
        String statementTotalAmount = df.format(doubleTotalAmount);
        bmstruckStatement.setStatementExcludTax(new BigDecimal(statementTotalAmount));
        return bmstruckStatementMapper.insertSelective(bmstruckStatement);
    }

    /**
     * 展示账单下的详单
     * @param mapVal
     * @return
     */
    @Override
    public List<Map<String, Object>> getSaleTruckStatementDetailsOrderList(Map<String, Object> mapVal) {
        return bmstruckStatementMapper.getSaleTruckStatementDetailsOrderList(mapVal);
    }
}
